<?php

/**
 * JCH Optimize - Aggregate and minify external resources for optmized downloads
 * 
 * @author Samuel Marshall <sdmarshall73@gmail.com>
 * @copyright Copyright (c) 2010 Samuel Marshall
 * @license GNU/GPLv3, See LICENSE file
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * If LICENSE file missing, see <http://www.gnu.org/licenses/>.
 */

class CodeAlfa_JchOptimize_Block_Form_Auto_OptimizeImages extends CodeAlfa_JchOptimize_Block_Form_Auto_Abstract
{

        protected function _getElementHtml(Varien_Data_Form_Element_Abstract $element)
        {
                $curl_enabled    = function_exists('curl_version') && curl_version();
                $allow_url_fopen = (bool) ini_get('allow_url_fopen');

                if ($curl_enabled && $allow_url_fopen)
                {
                        $ajax_url = Mage::helper('adminhtml')->getUrl('adminhtml/ajax/optimizeimages',
                                                                      array(
                                'module'     => 'jchoptimize',
                                'moduleName' => 'CodeAlfa_JchOptimize'
                                )
                        );

                        $filetree_ajax_url = Mage::helper('adminhtml')->getUrl('adminhtml/ajax/filetree',
                                                                               array(
                                'module'     => 'jchoptimize',
                                'moduleName' => 'CodeAlfa_JchOptimize'
                                )
                        );

                        $message = Mage::helper('jchoptimize')->__('Please open a directory to optimize images');

                        $html = "<script>
                                        var ajax_url = '{$ajax_url}'; 
                                        var filetree_ajax_url = '{$filetree_ajax_url}';        
                                        var message = '{$message}'; 
                                </script>";
                        $html .= '<div id="optimize-images-container" style="float:left; width: 100%;">'
                                . '<div id="file-tree-container" style="max-width:350px;float:left;"></div>';

                        $html .= parent::_getElementHtml($element);
                        $html .= '</div>';
                }
                else
                {
                        $message = !$allow_url_fopen ? Mage::helper('jchoptimize')->__('The PHP setting \'allow_url_fopen\' must be enabled for this feature')
                                        : '';
                        $message = !$curl_enabled ? Mage::helper('jchoptimize')->__('cURL is required for this feature but it\'s not enabled on this server.')
                                        : $message;

                        $html = '<div><ul class="messages"><li class="notice-msg"><ul><li><span>' . $message . '</span></li></ul></li></ul></div>';

                        $element->unsetData('tooltip');
                }

                return $html;
        }

        /**
         * 
         * @return string
         */
        protected function getButtons()
        {
                $page = Mage::helper('adminhtml')->getUrl('adminhtml/ajax/postresults',
                                                          array(
                        'module'     => 'jchoptimize',
                        'moduleName' => 'CodeAlfa_JchOptimize',
                        '_query'  => array('_' => time())                                         
                        )
                );

                $aButton              = array();
                $aButton[0]['link']   = '';
                $aButton[0]['icon']   = 'fa-compress';
                $aButton[0]['color']  = '#278EB1';
                $aButton[0]['text']   = 'Optimize Images';
                $aButton[0]['script'] = 'onclick="jchOptimizeImages(\'' . $page . '\'); return false;"';
                $aButton[0]['class']  = 'enabled';

                return $aButton;
        }

}
